@@ -2,7 +2,7 @@ svm <- function (x, ...)
   UseMethod ("svm")
 
 svm.formula <-
-function (formula, data=NULL, subset, na.action=na.fail, ...)
+function (formula, data = NULL, ..., subset, na.action = na.omit, scale = TRUE)
 {
   call <- match.call()
   if (!inherits(formula, "formula")) 
@@ -11,18 +11,28 @@ function (formula, data=NULL, subset, na.action=na.fail, ...)
   if (is.matrix(eval(m$data, parent.frame())))
     m$data <- as.data.frame(data)
   m$... <- NULL
+  m$scale <- NULL
   m[[1]] <- as.name("model.frame")
   m <- eval(m, parent.frame())
   Terms <- attr(m, "terms")
   attr(Terms, "intercept") <- 0
   x <- model.matrix(Terms, m)
   y <- model.extract(m, response)
-  ret <- svm.default (x, y, ...)
+  if (length(scale) == 1)
+    scale <- rep(scale, ncol(x))
+  if (any(scale)) {
+    remove <- unique(c(which(labels(Terms) %in% names(attr(x, "contrasts"))),
+                       which(!scale)
+                       )
+                     )
+    scale <- !attr(x, "assign") %in% remove
+  }
+  ret <- svm.default (x, y, scale = scale, ...)
   ret$call <- call
   ret$terms <- Terms
   if (!is.null(attr(m, "na.action"))) 
     ret$na.action <- attr(m, "na.action")
-  class (ret) <- c("svm.formula", class(ret))
+  class(ret) <- c("svm.formula", class(ret))
   return (ret)
 }
 
@@ -30,8 +40,6 @@ svm.default <-
 function (x,
           y         = NULL,
           scale     = TRUE,
-          subset,
-          na.action = na.fail,
           type      = NULL,
           kernel    = "radial",
           degree    = 3,
@@ -46,7 +54,9 @@ function (x,
           shrinking = TRUE,
           cross     = 0,
           fitted    = TRUE,
-          ...)
+          ...,
+          subset,
+          na.action = na.omit)
 {
   sparse  <- inherits(x, "matrix.csr")
   if (sparse) {
@@ -54,6 +64,7 @@ function (x,
       stop("Need SparseM package for handling of sparse structures!")
   }
 
+  ## NULL parameters?
   if(is.null(degree)) stop("`degree' must not be NULL!")
   if(is.null(gamma)) stop("`gamma' must not be NULL!")
   if(is.null(coef0)) stop("`coef0' must not be NULL!")
@@ -64,38 +75,43 @@ function (x,
   
   xhold   <- if (fitted) x else NA
   x.scale <- y.scale <- NULL
+  formula <- inherits(x, "svm.formula")
   
+  ## scaling, subsetting, and NA handling
   if (sparse) {
-    scale <- FALSE
-    na.fail(x)
+    scale <- rep(FALSE, ncol(x))
     if(!is.null(y)) na.fail(y)
     x <- t(t(x)) ## make shure that col-indices are sorted
   } else {
-    formula <- inherits(x, "svm.formula")
-    x <- if (is.vector(x)) t(t(x)) else as.matrix(x)
+    x <- as.matrix(x)
+
+    ## subsetting and na-handling for matrices
     if (!formula) {
       if (!missing(subset)) x <- x[subset,]
-      x <- if (is.null(y))
-        na.action(x)
+      if (is.null(y))
+        x <- na.action(x)
       else {
-        df <- data.frame(y, x)
-        df <- na.action(df)
+        df <- na.action(data.frame(y, x))
         y <- df[,1]
-        as.matrix(df[,-1])
+        x <- as.matrix(df[,-1])
       }
     }
-    
-    if (scale) {
-      co <- if (is.data.frame(x)) !sapply(x, var) else !apply(x, 2, var)
+
+    ## scaling
+    if (length(scale) == 1)
+      scale <- rep(scale, ncol(x))
+    if (any(scale)) {
+      co <- !apply(x, 2, var)
       if (any(co)) {
-        scale <- FALSE
+        scale <- rep(FALSE, ncol(x))
         warning(paste("Variable(s)",
                       paste("`",colnames(x)[co], "'", sep="", collapse=" and "),
                       "constant. Cannot scale data.")
                 )
       } else {
-        x <- scale(x)
-        x.scale <- attributes(x)[c("scaled:center","scaled:scale")]
+        xtmp <- scale(x[,scale])
+        x[,scale] <- xtmp
+        x.scale <- attributes(xtmp)[c("scaled:center","scaled:scale")]
         if (is.numeric(y)) {
           y <- scale(y)
           y.scale <- attributes(y)[c("scaled:center","scaled:scale")]
@@ -105,28 +121,34 @@ function (x,
     }
   }
 
+  ## determine model type
   if (is.null(type)) type <-
     if (is.null(y)) "one-classification"
     else if (is.factor(y)) "C-classification"
     else "eps-regression"
 
-  type <- pmatch (type, c("C-classification",
-                          "nu-classification",
-                          "one-classification",
-                          "eps-regression",
-                          "nu-regression"), 99) - 1
+  type <- pmatch(type, c("C-classification",
+                         "nu-classification",
+                         "one-classification",
+                         "eps-regression",
+                         "nu-regression"), 99) - 1
 
   if (type > 10) stop("wrong type specification!")
   
-  kernel <- pmatch (kernel, c("linear",
-                              "polynomial",
-                              "radial",
-                              "sigmoid"), 99) - 1
+  kernel <- pmatch(kernel, c("linear",
+                             "polynomial",
+                             "radial",
+                             "sigmoid"), 99) - 1
 
   if (kernel > 10) stop("wrong kernel specification!")
 
+  ## further parameter checks
+  nr <- nrow(x)
+  if (cross > nr)
+    stop("`cross' cannot exceed the number of observations!")
+
   if (!is.vector(y) && !is.factor (y) && type != 2) stop("y must be a vector or a factor.")
-  if (type != 2 && length(y) != nrow(x)) stop("x and y don't match.")
+  if (type != 2 && length(y) != nr) stop("x and y don't match.")
 
   if (cachesize < 0.1) cachesize <- 0.1
   
@@ -142,7 +164,7 @@ function (x,
       if (!is.null(class.weights)) {
         if (is.null(names(class.weights)))
           stop("Weights have to be specified along with their according level names !")
-        weightlabels <- match (names(class.weights),lev)
+        weightlabels <- match (names(class.weights), lev)
         if (any(is.na(weightlabels)))
           stop("At least one level name is missing or misspelled.")
       }
@@ -158,7 +180,7 @@ function (x,
   cret <- .C ("svmtrain",
               # data
               as.double  (if (sparse) x$ra else t(x)),
-              as.integer (nrow(x)), as.integer(ncol(x)),
+              as.integer (nr), as.integer(ncol(x)),
               as.double  (y),
 
               # sparse index info
@@ -186,11 +208,11 @@ function (x,
               # results
               nclasses = integer  (1), 
               nr       = integer  (1), # nr of support vectors
-              index    = integer  (nrow(x)),
+              index    = integer  (nr),
               labels   = integer  (nclass),
-              nSV      = integer  (nrow(x)),
+              nSV      = integer  (nr),
               rho      = double   (nclass * (nclass - 1) / 2),
-              coefs    = double   (nrow(x) * (nclass - 1)),
+              coefs    = double   (nr * (nclass - 1)),
               
               cresults = double   (cross),
               ctotal1  = double   (1),
@@ -236,7 +258,7 @@ function (x,
   # cross-validation-results
   if (cross > 0)    
     if (type > 2) {
-      scale.factor     <- if (scale) crossprod(y.scale$"scaled:scale") else 1;
+      scale.factor     <- if (any(scale)) crossprod(y.scale$"scaled:scale") else 1;
       ret$MSE          <- cret$cresults * scale.factor;
       ret$tot.MSE      <- cret$ctotal1  * scale.factor;
       ret$scorrcoeff   <- cret$ctotal2;
@@ -250,7 +272,7 @@ function (x,
   ret
 } 
 
-predict.svm <- function (object, newdata, ...) {
+predict.svm <- function (object, newdata, ..., na.action = na.omit) {
   if (missing(newdata))
     return(fitted(object))
 
@@ -266,15 +288,15 @@ predict.svm <- function (object, newdata, ...) {
       if(is.null(colnames(newdata)))
         colnames(newdata) <- colnames(object$SV)
       newdata <- model.matrix(delete.response(terms(object)),
-                              as.data.frame(newdata), na.action = na.omit)
-    } else if (!sparse) newdata <- as.matrix(newdata)
+                              as.data.frame(newdata), na.action = na.action)
+    } else if (!sparse) newdata <- na.action(as.matrix(newdata))
   }
 
-  if (object$scaled)
-    newdata <- scale(newdata,
-                     center = object$x.scale$"scaled:center",
-                     scale  = object$x.scale$"scaled:scale"
-                     )
+  if (any(object$scaled))
+    newdata[,object$scaled] <- scale(newdata[,object$scaled],
+                                    center = object$x.scale$"scaled:center",
+                                    scale  = object$x.scale$"scaled:scale"
+                                    )
 
   if (ncol(object$SV) != ncol(newdata)) stop ("test data does not match model !")
 
@@ -318,7 +340,7 @@ predict.svm <- function (object, newdata, ...) {
   else if (object$type == 2)
     # one-class-classification: return TRUE/FALSE
     ret == 1 
-  else if (object$scaled)
+  else if (any(object$scaled))
     # return raw values, possibly scaled back
     ret * object$y.scale$"scaled:scale" + object$y.scale$"scaled:center"
   else
@@ -416,17 +438,17 @@ plot.svm <- function(x, data, formula = NULL, fill = TRUE,
       names(lis)[1:2] <- colnames(sub)
       new <- expand.grid(lis)[,labels(terms(x))]
       preds <- predict(x, new)
-      filled.contour(xr, yr, matrix(codes(preds), nr = length(xr), byrow=TRUE),
+      filled.contour(xr, yr, matrix(as.numeric(preds), nr = length(xr), byrow=TRUE),
                      plot.axes = {
                        axis(1)
                        axis(2)
-                       colors <- codes(model.response(model.frame(x, data[-x$index,])))
+                       colors <- as.numeric(model.response(model.frame(x, data[-x$index,])))
                        points(formula, data = data[-x$index,], col = colors)
                        points(formula, data = data[x$index,], pch = "x", col = colors)
                      },
-                     levels = 1:(length(unique(codes(preds)))+1),
+                     levels = 1:(length(unique(as.numeric(preds)))+1),
                      key.axes = axis(4,
-                       1:length(unique(codes(preds)))+0.5,
+                       1:length(unique(as.numeric(preds)))+0.5,
                        labels = levels(preds)[unique(preds)], las = 3
                        ),
                      plot.title = title(main = "SVM classification plot",
@@ -435,7 +457,7 @@ plot.svm <- function(x, data, formula = NULL, fill = TRUE,
                      )
     } else {
       plot(formula, data = data, type = "n", ...)
-      colors <- codes(model.response(model.frame(m, data[-x$index,])))
+      colors <- as.numeric(model.response(model.frame(m, data[-x$index,])))
       points(formula, data = data[-x$index,], col = colors)
       points(formula, data = data[x$index,], pch = "x", col = colors)
     }
